package cytoPrefuse;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import cytoscape.Cytoscape;
import cytoscape.view.CytoscapeDesktop;

public class ControlPanel extends JPanel implements PropertyChangeListener, ListSelectionListener  {
	
	private static final long serialVersionUID = 1L;
	private Map displays = new HashMap();
	private float beta;
	private float labelDensity;
	private boolean showHulls;
	private CytoPrefuse currentVis;
	private JTable groupsTable;
	private JButton addGroup;
	private JButton deleteGroup;
	private static ControlPanel controlPanel = new ControlPanel();
	 
	public static ControlPanel instance() {
		return controlPanel;
	}
	
	public void addDisplay (CytoPrefuse vis, String title){
		displays.put(title,vis);
	}
	public void removeDisplay (String title){
		displays.remove(title);		
	}
	
	private ControlPanel(){
		Cytoscape.getDesktop().getSwingPropertyChangeSupport().addPropertyChangeListener(this);
		
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
	
		JLabel groupsLabel = new JLabel ("Groups", JLabel.CENTER);
		groupsLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	
		
		groupsTable = new JTable();
		groupsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);		
		groupsTable.getSelectionModel().addListSelectionListener(this);
		groupsTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked (MouseEvent e){
				int row = groupsTable.rowAtPoint(e.getPoint());
				if (row != -1){
					if (currentVis != null){
						currentVis.groupsManager.setCurrentGroup(row);
					}
				}			
			}
		});
				
		JScrollPane scrollPane = new JScrollPane(groupsTable);
		scrollPane.setPreferredSize(new Dimension(100, 150));
		add (scrollPane);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout( new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		
		
		addGroup = new JButton ("Add group");
		addGroup.setEnabled(false);
		addGroup.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (currentVis != null){
					currentVis.groupsManager.addGroup();
					groupsTable.setRowSelectionInterval(groupsTable.getRowCount()-1,groupsTable.getRowCount()-1);
					
				}
			}
		});

		deleteGroup = new JButton ("Delete group");
		deleteGroup.setEnabled(false);
		deleteGroup.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				int selectedGroup = groupsTable.getSelectedRow();
				GroupsManager groupsManager = (GroupsManager)groupsTable.getModel();
				groupsManager.removeGroup(selectedGroup);
				if (selectedGroup < groupsTable.getRowCount()){
					groupsTable.setRowSelectionInterval(selectedGroup, selectedGroup);
				}
			}
		});
		
		buttonPanel.add(addGroup);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(deleteGroup);

		add (Box.createRigidArea(new Dimension(0,5)));
		add (buttonPanel);
	
		add (Box.createRigidArea(new Dimension(0,50)));
		
		beta = 1.0f;
		JLabel edgeControllerLabel = new JLabel ("Edge curviness", JLabel.CENTER);
		edgeControllerLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		JSlider  edgeController  = new JSlider(JSlider.HORIZONTAL, 0, 10, 10);
		Hashtable labelTable = new Hashtable();
		labelTable.put (new Integer(0), new JLabel ("Straight"));
		labelTable.put (new Integer(10), new JLabel ("Bundled"));
		edgeController.setLabelTable(labelTable);
		edgeController.setPaintLabels(true);
		edgeController.setMajorTickSpacing(1);
		edgeController.setPaintTicks(true);
		edgeController.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider)e.getSource();
	            beta = source.getValue() *0.1f;
	            Iterator displaysIt = displays.values().iterator();
	            while (displaysIt.hasNext()){
	            	CytoPrefuse cytoPrefuse = (CytoPrefuse)displaysIt.next();
	            	cytoPrefuse.edgeBundler.setBeta(beta);
	            	cytoPrefuse.getVisualization().run("color");
				    
	            } 	
			}
			
		});
		
		add (edgeControllerLabel);
		add (edgeController);
		
		
		labelDensity = 0;
		JLabel labelDensityLabel = new JLabel ("Label Density", JLabel.CENTER);
		labelDensityLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		JSlider labelDensityController = new JSlider (JSlider.HORIZONTAL, 0, 100, 20);
		labelTable = new Hashtable();
		labelTable.put (new Integer(0), new JLabel ("Dense"));
		labelTable.put (new Integer(100), new JLabel ("Sparse"));
		labelDensityController.setLabelTable(labelTable);
		labelDensityController.setPaintLabels(true);
		labelDensityController.addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider)e.getSource();
	            labelDensity= source.getValue()/ 4000f;
	            Iterator displaysIt = displays.values().iterator();
	            while (displaysIt.hasNext()){
	            	CytoPrefuse cytoPrefuse = (CytoPrefuse)displaysIt.next();
	   		       cytoPrefuse.labelLayout.setExtraSpace(labelDensity);
	            }		     		
			}
			
		});
		add (Box.createRigidArea(new Dimension(0,10)));

		add (labelDensityLabel);
		add (labelDensityController);	
		add (Box.createRigidArea(new Dimension(0,10)));

		JCheckBox checkShowHulls = new JCheckBox("Show function groups");
		checkShowHulls.addItemListener(new ItemListener() {

			public void itemStateChanged(ItemEvent e) {
				showHulls =e.getStateChange() == ItemEvent.SELECTED; 
			    Iterator displaysIt = displays.values().iterator();
	            while (displaysIt.hasNext()){
	            	CytoPrefuse cytoPrefuse = (CytoPrefuse)displaysIt.next();
	            	cytoPrefuse.setShowHulls(showHulls);
	            }
			}
		});
		checkShowHulls.setSelected(true);
		
		add (checkShowHulls);
			
		
	}
	public boolean getShowHulls(){
		return showHulls;
	}

	
	public float getBeta(){
		return beta;		
	}
	
	public float getDensity() {
		return labelDensity;
	}
	
	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName() == CytoscapeDesktop.NETWORK_VIEW_FOCUSED) {
			currentVis = (CytoPrefuse) displays.get(Cytoscape.getCurrentNetworkView().getTitle());
			if (currentVis != null){
				groupsTable.setModel(currentVis.groupsManager);
			}
			addGroup.setEnabled(currentVis != null);
			deleteGroup.setEnabled(groupsTable.getRowCount() > 0);
		}		else if (e.getPropertyName() == CytoscapeDesktop.NETWORK_VIEW_DESTROYED){
			currentVis = null;
			groupsTable.setModel(new GroupsManager());
			addGroup.setEnabled(false);
			deleteGroup.setEnabled(false);
		}
	}

	


	public void valueChanged(ListSelectionEvent e) {
		ListSelectionModel lsm = (ListSelectionModel) e.getSource();
		if (!lsm.isSelectionEmpty() && currentVis != null){
			currentVis.groupsManager.setCurrentGroup(lsm.getMaxSelectionIndex());
		}
		deleteGroup.setEnabled(groupsTable.getRowCount()>0  && !lsm.isSelectionEmpty());
		
	}

	

}
