package cytoPrefuse;

import giny.model.GraphPerspective;
import giny.model.GraphPerspectiveChangeEvent;
import giny.model.GraphPerspectiveChangeListener;
import giny.view.EdgeView;
import giny.view.NodeView;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.OccasionalRepaintAction;
import prefuse.action.RepaintAction;
import prefuse.action.layout.AggregateLayout;
import prefuse.activity.Activity;
import prefuse.controls.BoundedPanControl;
import prefuse.controls.BoundedZoomControl;
import prefuse.controls.ClickToSelect;
import prefuse.controls.EdgeHighlightControl;
import prefuse.controls.GridDragger;
import prefuse.controls.IdleHighQualityRenderer;
import prefuse.controls.NeighborHighlightControl;
import prefuse.controls.NeighbourSelect;
import prefuse.controls.PopupMenuControl;
import prefuse.controls.RubberBandSelect;
import prefuse.controls.SplineBetaController;
import prefuse.controls.ZoomViewNeighbours;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.expression.AndPredicate;
import prefuse.data.expression.BooleanLiteral;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.ComparisonPredicate;
import prefuse.data.expression.NumericLiteral;
import prefuse.data.expression.Predicate;
import prefuse.data.tuple.BatchTupleSet;
import prefuse.data.tuple.DefaultTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.render.BundleEdgeRenderer;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EasyHoverShapeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.render.LineDividerRenderer;
import prefuse.render.PolygonRenderer;
import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.util.GraphicsLib;
import prefuse.util.PrefuseLib;
import prefuse.util.display.DisplayLib;
import prefuse.visual.AggregateItem;
import prefuse.visual.AggregateTable;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualGraph;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;
import prefuse.visual.sort.MyRenderSorter;
import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;
import cytoscape.data.CyAttributesUtils;
import cytoscape.data.SelectEvent;
import cytoscape.data.SelectEventListener;
import cytoscape.view.CyNetworkView;
import cytoscape.view.CytoscapeDesktop;
import cytoscape.visual.LineType;

public class CytoPrefuse extends Display implements SelectEventListener, GraphPerspectiveChangeListener{
	

	private static final String GROUP_LABELS = "groupLabels";
	public static final String GROUP_SIZE = "groupSize";
	private static final String RUBBER_BAND = "rubberBand";
	public static final String SPLINE_CONTROL = "splineControl";
	public static final String LABELS = "labels";
	public static final String LABEL_ITEM = "labelItem";
	public static final String SINGLE_HIGHLIGHT = "singleHighlight";
	private static final long serialVersionUID = -7249291917274938706L;
	public static final String BASE_FONT = "baseFont";
	public SearchGridLayout searchGridLayout;
	
	private static final Schema AGGR_SCHEMA = PrefuseLib.getVisualItemSchema();
	    static {
	        AGGR_SCHEMA.setDefault(VisualItem.INTERACTIVE, true);
	       }
	    
	    private static final Schema LABEL_SCHEMA = PrefuseLib.getVisualItemSchema();
	    static {
	        LABEL_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
	    }
	public static final String FUNCTION_NEIGHBOURS = "functionNeighbs";
	private static final String GRAPH = "graph";
	public static final String EDGES = "graph.edges";
	public static final String NODES = "graph.nodes";
	public static final String DNAINTERACTION = "DNAInteraction";
	public static final String CYTOSELECTED = "cytoselected";
	public static final String CYTOSCAPE_ID = "cytoscapeID";
	public static final String LABEL = "label";
	public static final String BIO_CATEGORY = "bioCategory";
	public static final String BIO_FUNCTION = "bioFunction";
	
	public static final String AGGR = "aggregates";
	public static final String DISPLAY_LAYER = "displayLayer";
	public static final String EDGE_SPLINE = "edgeSpline";
	public static final String EDGE_LENGTH = "edgeLength";
	public static final String SPLINE_BETA_ADJUSTED = "splineBetaAdjusted";
	public static final String BUNDLE_HUE = "bundleHue";
	public static final String SPLINE_OWNER = "splineOwner";
	public static final String EDGE_CLUSTERS = "edgeClusters";
	public static final String SELECTED = "selected";
	public static final String TEXT_COLOR = "textColor";
	public static final String FILL_COLOR = "fillColor";
	public static final String BORDER_COLOR = "borderColor";
	public static final String ON_SHORTEST_PATH = "onShortestPath";
	public static final String ON_PATHWAY = "onPathway";
	public static final String ANY_LAYER = "anyLayer";
	public static final String DOWNSTREAM_GENE ="downstreamGene";
	public static final String BIO_FUNCTION_UNKNOWN = "bioFunctionUnknown";
	private static final String LINE_DIVIDER = "lineDivider";
	public static final String LAYER = "layer";
	public static final String LINE_TYPE = "lineType";
	public static final String USER_INTEREST = "userInterest";
	private static final String CYTOSCAPE_DELETED = "cytoscapeDeleted";
	
	
	private ActionList layout;
	private Graph graph;
	private Map cytoscapeNodeKeyToPrefuseNode;
	private Map cytoscapeEdgeKeyToPrefuseEdge;
	private Map cytoscapeNodeIDToPrefuseNode;
	private Map cytoscapeEdgeIDToPrefuseEdge;
	private Visualization vis;
	public ColorManager colorManager;
	public GroupsManager groupsManager;
	public LabelLayout labelLayout;
	public EdgeBundler edgeBundler;
	
	private CyNetwork cyNetwork;
	private CyNetworkView cnv;
	private NeighbourSelect neighbourSelect;
	private NeighborHighlightControl neighbourHighlightControl;
	private ZoomViewNeighbours zoomViewNeighbours;
	private CytoPrefuse outerThis = this;
	private CytoscapeVisualMapper visMapper; 
	private CreateViewDialog layoutParameters;
	private JInternalFrame cytoscapeFrame;
	private Object[] singleItem = new Object[1];
	/**
	 * This constructor creates an action and adds it to the Plugins menu.
	 */
	public CytoPrefuse (CreateViewDialog layoutParameters) {

		this.layoutParameters = layoutParameters;
		saveLayoutParameters();
		
		createPrefuseView();
	}

	public int getHeight() {
		return super.getHeight();
	}
	private void createPrefuseView() {
		
		cyNetwork = Cytoscape.getCurrentNetwork();
		
		setHighQuality(true);

		createCytoscapeToolbarActions();
		
		final JInternalFrame iframe = new JInternalFrame( cyNetwork.getTitle(), true, true, true, true);
		final JDesktopPane cytoDesk = Cytoscape.getDesktop().getNetworkViewManager().getDesktopPane();
	   Cytoscape.createNetworkView(cyNetwork);
	   JInternalFrame[] allViews = cytoDesk.getAllFrames();
	   for (int i=0; i < allViews.length; i++){
		   if (allViews[i].getTitle() == cyNetwork.getTitle()){
			   cytoscapeFrame = allViews[i];
			   cytoscapeFrame.addInternalFrameListener(new InternalFrameAdapter(){

				public void internalFrameClosed(InternalFrameEvent arg0) {
					//Closing?  Close our window too
					iframe.dispose();
				}

				public void internalFrameActivated(InternalFrameEvent e) {
					//Activated?  Hide it and show our window instead
					cytoscapeFrame.hide();
					try {
						iframe.setIcon(false);
						iframe.show();						
						iframe.setSelected(true);
					} catch (PropertyVetoException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}				   
			   });
			   break;
		   }
	   }
	   cytoscapeFrame.hide();
	   cytoDesk.add(iframe);		
	   
	   iframe.getContentPane().add(this);
		ControlPanel.instance().addDisplay(this, cyNetwork.getTitle());
		iframe.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameActivated(InternalFrameEvent e) {
				
			Cytoscape.getDesktop().getSwingPropertyChangeSupport().firePropertyChange(new PropertyChangeEvent(Cytoscape.getDesktop(),
					CytoscapeDesktop.NETWORK_VIEW_FOCUSED, null, cyNetwork.getIdentifier()));
			}
			public void internalFrameClosed(InternalFrameEvent e) {
				vis.cancel("layout");
				JInternalFrame[] allViews = cytoDesk.getAllFrames();
				for (int i=0; i < allViews.length; i++){
				   if (allViews[i].getTitle() == cyNetwork.getTitle()){
					   CyNetworkView networkView = (CyNetworkView) Cytoscape.getNetworkViewMap().get(
								cyNetwork.getIdentifier());
					   if (networkView != null){
						   Cytoscape.destroyNetworkView(cyNetwork);
					   }
				   }
				}			
				ControlPanel.instance().removeDisplay(cyNetwork.getTitle());				
			}

			public void internalFrameIconified(InternalFrameEvent e) {
				vis.cancel("layout");
			}
			
			
		});
		vis = new Visualization();
		setVisualization(vis);
		
		loadNetwork();
		iframe.pack();
		iframe.setSize(400, 400);
		iframe.setVisible(true);
		
		try {
			iframe.setMaximum(true);
		} catch (PropertyVetoException e1) {
			//If the platform doesn't let us, who cares?  We're just maximizing the internal window.
			//So far this works on Windows, Solaris, and Mac.
		}
		
		setSize(iframe.getWidth(), iframe.getHeight()); 

	}
	private void saveLayoutParameters() {
		CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();	
		globalAttributes.setAttribute(Cytoscape.getCurrentNetwork().getIdentifier(), "cerebralParameters", layoutParameters.getSaveString());		
	}

	public void loadNetwork(){
		cyNetwork.addSelectEventListener(this);
		GraphPerspective network = (GraphPerspective)cyNetwork;
		network.addGraphPerspectiveChangeListener(this);
		
		cytoscapeNodeKeyToPrefuseNode = new HashMap();
		cytoscapeNodeIDToPrefuseNode = new HashMap();
		Table nodeTable = new Table();
		nodeTable.addColumn(LABEL, String.class);
		nodeTable.addColumn(CYTOSCAPE_ID, int.class);
		nodeTable.addColumn(CYTOSELECTED, boolean.class);
		nodeTable.addColumn(ANY_LAYER, boolean.class);		
		nodeTable.addColumn(BIO_CATEGORY, int.class);
		nodeTable.addColumn(BIO_FUNCTION, int.class);
		nodeTable.addColumn(BIO_FUNCTION_UNKNOWN, boolean.class);
		nodeTable.addColumn(FUNCTION_NEIGHBOURS, List.class);
		nodeTable.addColumn(DISPLAY_LAYER, int.class);
		nodeTable.addColumn(LABEL_ITEM, VisualItem.class);
		nodeTable.addColumn(EDGE_CLUSTERS, List.class, new ArrayList());
	    nodeTable.addColumn(BASE_FONT, Font.class);
	    nodeTable.addColumn(TEXT_COLOR, Color.class);
	    nodeTable.addColumn(FILL_COLOR, Color.class);
	    nodeTable.addColumn(BORDER_COLOR, Color.class);
		nodeTable.addColumn(ON_PATHWAY, boolean.class);
		nodeTable.addColumn(ON_SHORTEST_PATH, boolean.class);
		nodeTable.addColumn(DOWNSTREAM_GENE, boolean.class);
		nodeTable.addColumn(USER_INTEREST, boolean.class);
		nodeTable.addColumn(CYTOSCAPE_DELETED, boolean.class);
		

	  	 
		   
       
		Iterator nodesIt = network.nodesIterator();
		
		while (nodesIt.hasNext()) {
			giny.model.Node node = (giny.model.Node) nodesIt.next();

			int nodeId = nodeTable.addRow();
			cytoscapeNodeKeyToPrefuseNode.put((Object) node.getIdentifier(), new Integer(nodeId));
			cytoscapeNodeIDToPrefuseNode.put(new Integer(node.getRootGraphIndex()), new Integer(nodeId));
			
			nodeTable.setInt(nodeId, CYTOSCAPE_ID, node.getRootGraphIndex());
			nodeTable.setBoolean(nodeId, CYTOSELECTED, false);
		
			if (layoutParameters.placeAnyLayer(node)){
				nodeTable.setBoolean(nodeId, ANY_LAYER, true);					
			}else {
				nodeTable.setInt(nodeId, BIO_CATEGORY, layoutParameters.getLayer(node));			
			}
			if (layoutParameters.bioFunctionUnknown(node)){
				nodeTable.setBoolean(nodeId, BIO_FUNCTION_UNKNOWN, true);
			}else {				
				nodeTable.setInt(nodeId, BIO_FUNCTION, layoutParameters.getFunction(node) );
			}
		}
		moveDNAInteractionsToNucleus(network, nodeTable);

		cytoscapeEdgeKeyToPrefuseEdge = new HashMap();
		cytoscapeEdgeIDToPrefuseEdge = new HashMap();
		Table edgeTable = new Table();
		edgeTable.addColumn(Graph.DEFAULT_SOURCE_KEY, int.class);
		edgeTable.addColumn(Graph.DEFAULT_TARGET_KEY, int.class);
		edgeTable.addColumn(CYTOSCAPE_ID, int.class);
		edgeTable.addColumn(CYTOSELECTED, boolean.class);		
		edgeTable.addColumn(DNAINTERACTION, boolean.class);
		edgeTable.addColumn(SPLINE_CONTROL, List.class, new ArrayList());
		edgeTable.addColumn(SPLINE_BETA_ADJUSTED, List.class, new ArrayList());
		edgeTable.addColumn(SPLINE_OWNER, NodeItem.class);
		edgeTable.addColumn(EDGE_SPLINE, Shape.class);
		edgeTable.addColumn(EDGE_LENGTH, int.class);
		edgeTable.addColumn(FILL_COLOR, Color.class);
		edgeTable.addColumn(LINE_TYPE, LineType.class);
		edgeTable.addColumn(BUNDLE_HUE, float.class);
		edgeTable.addColumn(ON_PATHWAY, boolean.class);
		edgeTable.addColumn(ON_SHORTEST_PATH, boolean.class);
		edgeTable.addColumn(CYTOSCAPE_DELETED, boolean.class);

		Map edgeAttrMap = CyAttributesUtils.getAttribute(("interaction"),
				Cytoscape.getEdgeAttributes());

		Iterator edgesIt = network.edgesIterator();
		while (edgesIt.hasNext()) {
			giny.model.Edge edge = (giny.model.Edge) edgesIt.next();
		
			Integer source = (Integer) cytoscapeNodeKeyToPrefuseNode.get(edge.getSource()
					.getIdentifier());
			Integer target = (Integer) cytoscapeNodeKeyToPrefuseNode.get(edge.getTarget()
					.getIdentifier());
			
			int row = edgeTable.addRow();
			cytoscapeEdgeKeyToPrefuseEdge.put(edge.getIdentifier(), new Integer(row));
			cytoscapeEdgeIDToPrefuseEdge.put(new Integer (edge.getRootGraphIndex()), new Integer(row));

			edgeTable.setBoolean(row, DNAINTERACTION, false);
			edgeTable.setInt(row, CYTOSCAPE_ID, edge.getRootGraphIndex() );
			if (edgeAttrMap != null) {
				Object att = edgeAttrMap.get(edge.getIdentifier());
				if (att instanceof String) {
					String interaction = (String) att;
					if ("pd".compareToIgnoreCase(interaction) == 0) {
						edgeTable.setBoolean(row, DNAINTERACTION, true);
					}
				}
			}
			
			edgeTable.setInt(row, Graph.DEFAULT_SOURCE_KEY, source.intValue());
			edgeTable.setInt(row, Graph.DEFAULT_TARGET_KEY, target.intValue());		
		}
		graph = new Graph(nodeTable, edgeTable, false, null,
				Graph.DEFAULT_SOURCE_KEY, Graph.DEFAULT_TARGET_KEY);

		
	
		
		VisualGraph vg = vis.addGraph(GRAPH, graph);
		vis.setInteractive(EDGES, null, true);		 
		vis.addDecorators(LABELS, NODES, LABEL_SCHEMA);
		
		vis.addFocusGroup(SINGLE_HIGHLIGHT, new DefaultTupleSet());
		BatchTupleSet selectedItems = new BatchTupleSet();
		vis.addFocusGroup(SELECTED, selectedItems);
		
		//Create table for storing aggregate hull locations
		AggregateTable at = vis.addAggregates(AGGR,AGGR_SCHEMA);		
		at.addColumn(LABEL, String.class);
		at.addColumn(BIO_FUNCTION, int.class);
		at.addColumn(GROUP_SIZE, int.class);
	    at.addColumn(VisualItem.POLYGON, float[].class);
		
	
	 	TupleSet nodes = vis.getGroup( NODES );

		// Map exisitng cytoscape positions to initial positions of prefuse
		cnv = Cytoscape.getCurrentNetworkView();
		if (cnv != null){
			nodesIt = network.nodesIterator();
			while (nodesIt.hasNext()) {
				giny.model.Node cytoNode = (giny.model.Node) nodesIt.next();
				Integer id = (Integer) cytoscapeNodeKeyToPrefuseNode.get(cytoNode.getIdentifier());
				Node node = graph.getNode(id.intValue());
				NodeItem nodeItem = (NodeItem) vis.getVisualItem(NODES,
						node);
	
		
				NodeView nodeView = cnv.getNodeView(cytoNode);
				if (nodeView!= null){
					nodeItem.setX(nodeView.getXPosition());
					nodeItem.setEndX(nodeView.getXPosition());
					nodeItem.setY(nodeView.getYPosition());
					nodeItem.setEndY(nodeView.getYPosition());
					if (nodeView.isSelected()){
						nodeItem.setBoolean(CYTOSELECTED, true);		
						selectedItems.addTuple(nodeItem);
	
					}
				}
				edgesIt = network.edgesIterator();
				while (edgesIt.hasNext()){
					giny.model.Edge cytoEdge = (giny.model.Edge)edgesIt.next();
					id = (Integer) cytoscapeEdgeKeyToPrefuseEdge.get(cytoEdge.getIdentifier());
					prefuse.data.Edge edge = graph.getEdge(id.intValue());
					EdgeItem edgeItem = (EdgeItem) vis.getVisualItem(EDGES,
							edge);
					EdgeView edgeView = cnv.getEdgeView(cytoEdge);
					if (edgeView != null){
						if (edgeView.isSelected()){
							edgeItem.setBoolean(CYTOSELECTED, true);
							selectedItems.addTuple(edgeItem);
						}
					}
			
				}
			}			
			int[] minYs = new int[layoutParameters.getNumLayers()];
			Arrays.fill(minYs, Integer.MAX_VALUE);
			nodesIt = nodes.tuples();			
			while (nodesIt.hasNext()){
				NodeItem nodeItem = (NodeItem)nodesIt.next();
				int y = (int) nodeItem.getY();
				if (!nodeItem.getBoolean(ANY_LAYER)){
					int bioCategory = nodeItem.getInt(BIO_CATEGORY);
					if (y < minYs[bioCategory]){
						minYs[bioCategory]= y;
					}
				}
			}
			nodesIt = nodes.tuples();			
			while (nodesIt.hasNext()){
				NodeItem nodeItem = (NodeItem)nodesIt.next();
				int bioCategory = nodeItem.getInt(BIO_CATEGORY);
				boolean anyLayer = nodeItem.getBoolean(ANY_LAYER);
				if (!anyLayer){
					nodeItem.setInt(DISPLAY_LAYER, bioCategory);
				}else {
					int y = (int) nodeItem.getY();
					int layer = 0;
					for (int i= 0; i < minYs.length; i++){
						if (y > minYs[i]){
							layer = i;
						}
					}
					nodeItem.setInt(DISPLAY_LAYER, layer);
				}
			}
			
		}
   
		for (int j=0; j< layoutParameters.getNumLayers(); j++){
			 for (int i=0; i < layoutParameters.getNumFunctions(); i++) {
				   Predicate functionGroup = new AndPredicate(
						   new ComparisonPredicate(ComparisonPredicate.EQ, new ColumnExpression(CytoPrefuse.BIO_FUNCTION), new NumericLiteral(i) ),
						   new ComparisonPredicate (ComparisonPredicate.EQ, new ColumnExpression(CytoPrefuse.BIO_CATEGORY), new  NumericLiteral(j)));
				   Iterator functionGroupIt = nodes.tuples( functionGroup);
				   List group = new LinkedList();
				   while (functionGroupIt.hasNext()) {
					   NodeItem node = (NodeItem) functionGroupIt.next();
					   group.add(node);
					   node.set (FUNCTION_NEIGHBOURS, group);
				   }
			   }
		}
		
   
	   for (int i=0; i <layoutParameters.getNumFunctions(); i++) {
		   Predicate functionGroup = new AndPredicate(
				   new ComparisonPredicate(ComparisonPredicate.EQ, new ColumnExpression(CytoPrefuse.BIO_FUNCTION), new NumericLiteral(i) ),
				   new ComparisonPredicate (ComparisonPredicate.EQ, new ColumnExpression(CytoPrefuse.DOWNSTREAM_GENE), new BooleanLiteral(true)));
		   Iterator functionGroupIt = nodes.tuples( functionGroup);
		   AggregateItem ai = (AggregateItem) at.addItem();
		   ai.setFillColor( ColorLib.rgba(125,194,223,50));
		   String label = layoutParameters.getFunctionName(i);		
		   ai.setString(LABEL,	   label.replace(" ", "\n") );
		   ai.setInt(BIO_FUNCTION, i);
		   List group = new LinkedList();
		   while (functionGroupIt.hasNext()) {
			   NodeItem node = (NodeItem) functionGroupIt.next();
			   group.add(node);
			   node.set (FUNCTION_NEIGHBOURS, group);
			   ai.addItem( node);
		   }
		   ai.setInt(GROUP_SIZE, group.size());
	   }
   
	   
		vis.addDecorators(GROUP_LABELS, AGGR, LABEL_SCHEMA);
		
		
		Table rubberBandTable = new Table();
		rubberBandTable.addColumn(VisualItem.POLYGON, float[].class);			
		rubberBandTable.addRow();
		vis.add( RUBBER_BAND, rubberBandTable);		
		VisualItem rubberBand = (VisualItem) vis.getVisualGroup(RUBBER_BAND).tuples().next();		
		rubberBand.set(VisualItem.POLYGON, new float[8]);
		rubberBand.setStrokeColor(ColorLib.color(ColorLib.getColor(255,0,0)));
			
		
		Table lineDividerTable = new Table();
		lineDividerTable.addColumn (LABEL, String.class);
		lineDividerTable.addColumn (LAYER, int.class);
		ArrayList allLabels = new ArrayList();
		for (int i=0; i < layoutParameters.getNumLayers(); i++){
			int row = lineDividerTable.addRow();
			String region =layoutParameters.getLayerName(i);
			lineDividerTable.setString(row, LABEL, region);
			lineDividerTable.setInt(row, LAYER, i);
			allLabels.add(region);
		}
		vis.add (LINE_DIVIDER, lineDividerTable);
		
		
		
		// -- 3. the renderers and renderer factory ---------------------------

		colorManager = new ColorManager();
		
		 Renderer polyR = new PolygonRenderer(Constants.POLY_TYPE_CURVE);
	    ((PolygonRenderer)polyR).setCurveSlack(0.15f);
	        
		 EasyHoverShapeRenderer nodeRenderer = new EasyHoverShapeRenderer(this);
		 nodeRenderer.setBaseSize(20);
		 
		 BundleEdgeRenderer edgeRenderer = new BundleEdgeRenderer();
	     DefaultRendererFactory drf = new DefaultRendererFactory(nodeRenderer, edgeRenderer);
	     
	 	 Renderer rubberBandRenderer = new PolygonRenderer(Constants.POLY_TYPE_LINE);
	 	 LineDividerRenderer lineDividerRenderer = new LineDividerRenderer(vis, allLabels, layoutParameters.displayDownstreamGenes(), layoutParameters.getDownstreamLabel());
		 
	 	searchGridLayout = new SearchGridLayout(vg, "layout", null, layoutParameters.getNumLayers(), lineDividerRenderer);
		
	 	 SearchGridLayoutPostProcessAction searchGridLayoutPostProcessAction = new SearchGridLayoutPostProcessAction(searchGridLayout, vg, null);
		
		 drf.add(new InGroupPredicate(AGGR), polyR);
	     drf.add(new InGroupPredicate(LABELS), new LabelRenderer());
	     drf.add(new InGroupPredicate(GROUP_LABELS), new LabelRenderer()) ;
	     drf.add(new InGroupPredicate(RUBBER_BAND), rubberBandRenderer);
	     drf.add(new InGroupPredicate(LINE_DIVIDER), lineDividerRenderer);
	     vis.setRendererFactory(drf);
	    	
		// -- the processing actions ---------------------------------------

	

		
		DecoratorItem labelOrder[] = new DecoratorItem[vis.getGroup(LABELS).getTupleCount() +
		                                               vis.getGroup(GROUP_LABELS).getTupleCount()];
		
	   Iterator iter = vis.items(LABELS);	        
		int i =0;
	   while (iter.hasNext()) {
			labelOrder[i++] = (DecoratorItem) (iter.next());
		}
	   
	   iter = vis.items(LABELS);	        
	   while (iter.hasNext()) {
		   DecoratorItem di = (DecoratorItem) (iter.next());
		   NodeItem node = (NodeItem) di.getDecoratedItem();
		   node.set(LABEL_ITEM, di);
	   }	   
	   
	   iter = vis.items(GROUP_LABELS);	        
		while (iter.hasNext()) {
			labelOrder[i++] = (DecoratorItem) (iter.next());
		}
	   
	
	   
		labelLayout = new LabelLayout(labelOrder,  new LabelSorter(), this);
		labelLayout.setExtraSpace(ControlPanel.instance().getDensity());
	 		    
		layout = new ActionList(Activity.INFINITY);
		
		layout.add(searchGridLayout);					
		layout.add(new AggregateLayout(AGGR, 20));	       
		layout.add(new OccasionalRepaintAction(20));
		
		
		ActionList aggregateLayout = new ActionList ();
		aggregateLayout.add (new AggregateLayout(AGGR,1));
		

		visMapper =new CytoscapeVisualMapper(vis, cyNetwork, colorManager);
		Cytoscape.getVisualMappingManager().addChangeListener(visMapper);
	

		// add the actions to the visualization
		colorManager.createColorRules(vis, labelLayout, edgeTable);
		
		vis.putAction("repaint", new RepaintAction());
		vis.putAction("layout", layout);
		vis.putAction("aggLayout", aggregateLayout);
		//vis.putAction("delayHighQuality", new SetHighQualityAndRepaintAction(this));
		vis.putAction("delayHighQuality", new ActionList());
		vis.putAction("layoutPostProcess", searchGridLayoutPostProcessAction);
		vis.putAction("applyVizMapper", visMapper);
		
	

		edgeBundler = new EdgeBundler(vis.getVisualGroup(NODES), vis.getVisualGroup(EDGES));
		edgeBundler.recomputeLocations();
		edgeBundler.setBeta(ControlPanel.instance().getBeta());
		setItemSorter(new MyRenderSorter(edgeTable));
	
		createDisplayControls( vg, labelLayout, edgeBundler, edgeTable, rubberBand);
		
		setShowHulls(ControlPanel.instance().getShowHulls());
	   

		groupsManager = new GroupsManager(vis);


		
		// assign the colors
		vis.run("color");		
		

		vis.run("zoomToFit");
	}

	private void createCytoscapeToolbarActions() {
		//Try to add a listener for the toolbars
		Component[] toolbarButtons =Cytoscape.getDesktop().getCyMenus().getToolBar().getComponents();
		for (int i=0; i < toolbarButtons.length;i++){
		
			if (toolbarButtons[i] instanceof JButton) {
				JButton button = (JButton)toolbarButtons[i];
				String tooltipText = button.getToolTipText();
				if (tooltipText != null) {							 
					if (button.getToolTipText().compareToIgnoreCase("Zoom Out")==0){
						button.addActionListener(new ActionListener() {	
							public void actionPerformed(ActionEvent e) {
							    Point2D down = new Point2D.Float(getWidth()/2, getHeight()/2);								   
								zoom(down, 0.9);			
								repaint();
								
							}
						});
					}
					if (button.getToolTipText().compareToIgnoreCase("Zoom In")==0){
						button.addActionListener(new ActionListener() {	
							public void actionPerformed(ActionEvent e) {
								Point2D down = new Point2D.Float(getWidth()/2, getHeight()/2);										   
								zoom(down, 1.1);			
								repaint();								
							}
						});						
					}
					
					if (button.getToolTipText().compareToIgnoreCase("Zoom out to display all of current Network")==0){
						button.addActionListener(new ActionListener() {	
							public void actionPerformed(ActionEvent e) {
								 if (!outerThis.isTranformInProgress()){
									 vis.getDisplay(0).setHighQuality(false);									    
								     Rectangle2D bounds = vis.getBounds(NODES);
							         GraphicsLib.expand(bounds, 50 + (int)(1/getScale()));
							         DisplayLib.fitViewToBounds(outerThis, bounds, 2000);
							         vis.runAfter("delayHighQuality", 2000);
								}
							}
						});
					}
					if (button.getToolTipText().compareToIgnoreCase("Zoom Selected Region")==0){
						button.addActionListener(new ActionListener() {	
							public void actionPerformed(ActionEvent e) {
								 TupleSet selectedItems = vis.getFocusGroup(SELECTED);
								 Iterator it = selectedItems.tuples();
								 Rectangle2D bounds = null;
								 while (it.hasNext() ) {
									 VisualItem vi = (VisualItem)it.next();
									 if (vi instanceof NodeItem) {
										 if (bounds == null){
											 Rectangle2D initBounds = vi.getBounds();
											 bounds = new Rectangle2D.Double(initBounds.getX(), initBounds.getY(), initBounds.getWidth(), initBounds.getHeight());
										 }else{
											 Rectangle2D.union(bounds,vi.getBounds(), bounds);
										 }
									 }
								 }
								 if (bounds != null) {
									 vis.getDisplay(0).setHighQuality(false);
							         GraphicsLib.expand(bounds, 50 + (int)(1/getScale()));
							         DisplayLib.fitViewToBounds(outerThis, bounds, 2000);
							         vis.runAfter("delayHighQuality", 2000);
								 }
							}
						});
					}	
					if (button.getToolTipText().equalsIgnoreCase("Show all Nodes and Edges (unhiding as necessary)")){
						button.addActionListener(new ActionListener() {

							public void actionPerformed(ActionEvent e) {
								Iterator nodesIt = vis.getVisualGroup(NODES).tuples();
								while (nodesIt.hasNext()) {
									NodeItem nodeItem = (NodeItem)nodesIt.next();
									if (!nodeItem.getBoolean(CYTOSCAPE_DELETED)){
										nodeItem.setVisible(true);
									}
								}
								Iterator edgesIt = vis.getVisualGroup(EDGES).tuples();
								while (edgesIt.hasNext()) {
									EdgeItem edgeItem = (EdgeItem)edgesIt.next();
									if (!edgeItem.getBoolean(CYTOSCAPE_DELETED)){
										edgeItem.setVisible(true);
									}
								}
								showAggregatesIfAnyMemberVisible();
								vis.run("color");
							}
						});
					}
					if (button.getToolTipText().equalsIgnoreCase("Hide Selected Region")){
						button.addActionListener(new ActionListener() {

							public void actionPerformed(ActionEvent e) {
								Iterator nodesIt = vis.getVisualGroup(NODES).tuples();
								while (nodesIt.hasNext()) {
									NodeItem nodeItem = (NodeItem)nodesIt.next();
									if (nodeItem.getBoolean(CYTOSELECTED)){
										nodeItem.setVisible(false);
									}
								}
								Iterator edgesIt = vis.getVisualGroup(EDGES).tuples();
								while (edgesIt.hasNext()) {
									EdgeItem edgeItem = (EdgeItem)edgesIt.next();
									if (edgeItem.getBoolean(CYTOSELECTED)){										
										edgeItem.setVisible(false);
									}
								}
								hideAggregatesIfAllMembersHidden();
						
								vis.run("color");
							}

						
						});
					}
				}
			}
		}
	}

	
	private void createDisplayControls(VisualGraph vg, LabelLayout ll, EdgeBundler edgeBundler, Table edges, VisualItem rubberBand) {
		addControlListener(new GridDragger(vis, vg.getNodes(), edgeBundler));
		//addControlListener(new PathwayChooser(vg, vis));
		addControlListener(new BoundedPanControl(prefuse.controls.Control.MIDDLE_MOUSE_BUTTON, NODES));
		
		addPaintListener(ll);
		
		RubberBandSelect rubberBandSelect = new RubberBandSelect(rubberBand);
		addControlListener(rubberBandSelect);
		addControlListener(new BoundedZoomControl(NODES));
		addControlListener(new SaveAction());
		addControlListener( new SplineBetaController(edgeBundler));
		
		neighbourHighlightControl = new NeighborHighlightControl("color");
		addControlListener(neighbourHighlightControl);
		addControlListener(new EdgeHighlightControl("color"));
		zoomViewNeighbours = new ZoomViewNeighbours(vis, vis.getFocusGroup(SINGLE_HIGHLIGHT)); 
		addControlListener (zoomViewNeighbours);
		
		neighbourSelect =new NeighbourSelect(vis.getFocusGroup(SINGLE_HIGHLIGHT));
		addControlListener(neighbourSelect);
		addControlListener(new ClickToSelect());
		addControlListener(new PopupMenuControl(this,vis));
	
		TupleSet focusGroup = vis.getGroup(SELECTED);
		focusGroup.addTupleSetListener(new TupleSetListener() {
			public void tupleSetChanged(TupleSet ts, Tuple[] add, Tuple[] rem) {
				selectInCytoscape(rem, false);
				selectInCytoscape(add, true);	
				vis.run("color");
			}
		});
		
		if (vg.getNodeCount() + vg.getEdgeCount() > 300) {
			addControlListener(new IdleHighQualityRenderer(this));
		}
	}
	
	public void selectInCytoscape (Object[] visualItems, boolean selectState){
		List cytoNodes = new ArrayList();
		List cytoEdges = new ArrayList();
		for (int i=0; i < visualItems.length; i++) {
			Object obj = visualItems[i];
			if (obj instanceof VisualItem) {
				VisualItem vi= (VisualItem) obj;
				if (vi.isVisible()  && !vi.getBoolean(CYTOSCAPE_DELETED)){							
					if (vi instanceof NodeItem){
						NodeItem nodeItem = (NodeItem) vi;
						int id = nodeItem.getInt(CYTOSCAPE_ID);
						giny.model.Node node = cyNetwork.getNode(id);
						cytoNodes.add(node);	
					}else if (vi instanceof EdgeItem) {
						EdgeItem edgeItem = (EdgeItem) vi;
						int id = edgeItem.getInt(CYTOSCAPE_ID);
						giny.model.Edge edge = cyNetwork.getEdge(id);
						cytoEdges.add(edge);
					}
				}
			}else {
				System.out.println ("I don't know how to deal with this item.");
			}
		}
		if (cytoNodes.size() > 0 || cytoEdges.size() > 0){
		cyNetwork.setSelectedNodeState(cytoNodes, selectState);
		cyNetwork.setSelectedEdgeState(cytoEdges, selectState);
		}
	}


	private void moveDNAInteractionsToNucleus(GraphPerspective network,
			Table nodeTable) {
	
		if (layoutParameters.downstreamIsEdgeAttribute()){
			Iterator edgeIt = network.edgesIterator();
			while (edgeIt.hasNext()) {
				giny.model.Edge edge = (giny.model.Edge) edgeIt.next();
				if (layoutParameters.isDownstreamGene(edge)){
					giny.model.Node target = edge.getTarget();
					Integer row = (Integer) cytoscapeNodeKeyToPrefuseNode.get(target
							.getIdentifier());
					nodeTable.setBoolean(row.intValue(), DOWNSTREAM_GENE, true);							
				}			
			}		
		}else{
			Iterator nodeIt = network.nodesIterator();
			while (nodeIt.hasNext()) {
				giny.model.Node node = (giny.model.Node) nodeIt.next();
				if (layoutParameters.isDownstreamGene(node)){
					Integer row = (Integer) cytoscapeNodeKeyToPrefuseNode.get(node.getIdentifier());
					nodeTable.setBoolean(row.intValue(), DOWNSTREAM_GENE, true);							
				}			
			}		
		
		}
	}
	

	public void onSelectEvent(SelectEvent se) {

		//Cytoscape generated select event
		boolean selecting = se.getEventType();
		if (se.getTargetType() == SelectEvent.NODE_SET) {
			Collection cytoNodes = (Collection) se.getTarget();
			
			Iterator it = cytoNodes.iterator();
			while (it.hasNext()) {
				giny.model.Node cytoNode = (giny.model.Node) it.next();
				Integer key = (Integer) cytoscapeNodeKeyToPrefuseNode.get(cytoNode
						.getIdentifier());
				if (key != null) {
					prefuse.data.Node node = graph.getNodeFromKey(key.intValue());
					node.setBoolean(CYTOSELECTED, selecting);
				}
			}
		}else if (se.getTargetType() == SelectEvent.SINGLE_NODE) {
			giny.model.Node cytoNode = (giny.model.Node) se.getTarget();
			Integer key = (Integer) cytoscapeNodeKeyToPrefuseNode.get(cytoNode.getIdentifier());
			if (key != null){
				prefuse.data.Node node = graph.getNodeFromKey(key.intValue());				
				node.setBoolean(CYTOSELECTED, selecting);
			}
		}else if (se.getTargetType() == SelectEvent.EDGE_SET){
			Collection cytoEdges = (Collection) se.getTarget();
			Iterator it = cytoEdges.iterator();
			while (it.hasNext()) {
				giny.model.Edge cytoEdge = (giny.model.Edge) it.next();
				Integer key = (Integer) cytoscapeEdgeKeyToPrefuseEdge.get(cytoEdge.getIdentifier());
				if (key != null){
					prefuse.data.Edge edge = graph.getEdge(key.intValue());
					edge.setBoolean(CYTOSELECTED, selecting);
				}
			}
		}else if (se.getTargetType() == SelectEvent.SINGLE_EDGE) {
			giny.model.Edge cytoEdge = (giny.model.Edge) se.getTarget();
			Integer key = (Integer) cytoscapeEdgeKeyToPrefuseEdge.get(cytoEdge.getIdentifier());
			if (key != null){
				prefuse.data.Edge edge = graph.getEdge(key.intValue());				
				edge.setBoolean(CYTOSELECTED, selecting);
			}
		}
		
		vis.run(("color"));
			
			
	}

	public void setCytoscapePostionsFromLayout(TupleSet nodeItems) {
		if (cnv != null){
	
			Iterator it = nodeItems.tuples();
			while (it.hasNext()){
				NodeItem nodeItem = (NodeItem)it.next();
				int cytoIndex = nodeItem.getInt(CytoPrefuse.CYTOSCAPE_ID);
				giny.model.Node cytoNode = cyNetwork.getNode(cytoIndex);
				NodeView nodeView = cnv.getNodeView(cytoNode);
				if (nodeView!= null){
					nodeView.setXPosition(nodeItem.getX());
					nodeView.setYPosition(nodeItem.getY());
				}	
			}
		}
	
	}

	public void setShowHulls(boolean show) {
		Iterator aggregateIt = vis.getVisualGroup(AGGR).tuples();
		while (aggregateIt.hasNext()) {
			AggregateItem aggItem = (AggregateItem)aggregateIt.next();
			aggItem.setVisible(show);
		}
		vis.run("aggLayout");
		vis.run("color");
	}

	public NodeItem getNodeItemFromCytoID(String objectKey) {
		Integer id = (Integer) cytoscapeNodeKeyToPrefuseNode.get(objectKey);
		Node node = graph.getNode(id.intValue());
		NodeItem nodeItem = (NodeItem) vis.getVisualItem(NODES,
				node);
		return nodeItem;
	}
	
	public NodeItem getNodeItemFromCytoID(int id) {
		Integer prefuseId = (Integer) cytoscapeNodeIDToPrefuseNode.get(new Integer(id));
		if (prefuseId != null){
			Node node = graph.getNode(prefuseId.intValue());
			NodeItem nodeItem = (NodeItem)vis.getVisualItem(NODES, node);
			return nodeItem;
		}
		return null;
	}
	
	public EdgeItem getEdgeItemFromCytoID (int id){
		Integer prefuseId = (Integer) cytoscapeEdgeIDToPrefuseEdge.get(new Integer(id));
		if (prefuseId != null) {
			Edge edge = graph.getEdge(prefuseId.intValue());
			EdgeItem edgeItem = (EdgeItem)vis.getVisualItem(EDGES, edge);
			return edgeItem;
		}
		return null;
	}
	

	public void graphPerspectiveChanged(GraphPerspectiveChangeEvent e) {
		
		//Cytoscape lies.  These events don't get generated becuase edges/nodes are hidden
		//They get generated because these edges/nodes are removed.
		switch (e.getType()){
			case GraphPerspectiveChangeEvent.EDGES_HIDDEN_TYPE: 
			{
				int [] edgeIndices = e.getHiddenEdgeIndices();
				for (int i=0; i <edgeIndices.length; i++) {
					EdgeItem edgeItem = getEdgeItemFromCytoID(edgeIndices[i]);
					if (edgeItem != null){
						edgeItem.setVisible(false);
						edgeItem.setBoolean(CYTOSCAPE_DELETED, true);
					}
				}
				break;		
			}
			case GraphPerspectiveChangeEvent.NODES_HIDDEN_TYPE:
			{
				int [] nodeIndices = e.getHiddenNodeIndices();
				for (int i=0; i <nodeIndices.length; i++) {
					NodeItem nodeItem = getNodeItemFromCytoID(nodeIndices[i]);
				
					if (nodeItem != null) {
						nodeItem.setVisible(false);
						nodeItem.setBoolean(CYTOSCAPE_DELETED, true);
						hideAggregatesIfAllMembersHidden();
					}
				}
				break;
			}
			case GraphPerspectiveChangeEvent.NODES_RESTORED_TYPE:
			{
				int [] nodeIndices = e.getRestoredNodeIndices();
				for (int i=0; i <nodeIndices.length; i++) {
					NodeItem nodeItem = getNodeItemFromCytoID(nodeIndices[i]);
				
					if (nodeItem != null) {
						nodeItem.setVisible(true);
						nodeItem.setBoolean(CYTOSCAPE_DELETED, false);
						showAggregatesIfAnyMemberVisible();
					}
				}
				break;
			}
			case GraphPerspectiveChangeEvent.EDGES_RESTORED_TYPE: 
			{
				int [] edgeIndices = e.getRestoredEdgeIndices();
				for (int i=0; i <edgeIndices.length; i++) {
					EdgeItem edgeItem = getEdgeItemFromCytoID(edgeIndices[i]);
					if (edgeItem != null){
						edgeItem.setVisible(true);
						edgeItem.setBoolean(CYTOSCAPE_DELETED, false);
					}
				}
				break;		
			
			}
			default:
				System.out.println ("Received an unexpected GraphPerspectiveChangeEvent");
		}
			
	}

	private void showAggregatesIfAnyMemberVisible() {
		Iterator aggregateIt = vis.getVisualGroup(AGGR).tuples();
		while (aggregateIt.hasNext()) {
			AggregateItem aggItem = (AggregateItem)aggregateIt.next();
			boolean atLeastOneVisible = false;
			Iterator items = aggItem.items();
			while (items.hasNext()){
				VisualItem item = (VisualItem) items.next();
				if (item.isVisible()){
					atLeastOneVisible = true;
					break;
				}
			}
			if (atLeastOneVisible) {
				aggItem.setVisible(true);
			}
		}

		vis.run("aggLayout");
		vis.run("color");
	}
	private void hideAggregatesIfAllMembersHidden() {
		//See if we hid everything in an aggregate group
		Iterator aggregateIt = vis.getVisualGroup(AGGR).tuples();
		while (aggregateIt.hasNext()) {
			AggregateItem aggItem = (AggregateItem)aggregateIt.next();
			Iterator membersIt = aggItem.items();
			boolean allInvisible = true;
			while (membersIt.hasNext()){
				NodeItem nodeItem = (NodeItem)membersIt.next();
				if (nodeItem.isVisible()){
					allInvisible = false;
					break;
				}
			}
			if (allInvisible){
				aggItem.setVisible(false);
			}
		}
		vis.run("aggLayout");
		vis.run("color");
	}

	public void deselectAll() {

		List deselectItems = new ArrayList();
		Iterator it = vis.getVisualGroup(CytoPrefuse.NODES).tuples();
		while (it.hasNext()){
			 NodeItem node = (NodeItem)it.next();
			 if (node.getBoolean(CYTOSELECTED)){
				 node.setBoolean(CYTOSELECTED, false);
				 deselectItems.add(node);
			 }
		}
		it = vis.getVisualGroup(CytoPrefuse.EDGES).tuples();
		while (it.hasNext()){
			 EdgeItem edge = (EdgeItem)it.next();
			 if (edge.getBoolean(CYTOSELECTED)){
				 edge.setBoolean(CYTOSELECTED,false);
				 deselectItems.add(edge);
			 }
		}
		selectInCytoscape(deselectItems.toArray(), false);
	}
	
	public void deselectSingle(VisualItem item){
		singleItem[0] = item;
		selectInCytoscape(singleItem, false);
	}




	

}