package prefuse.controls;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;

import javax.swing.SwingUtilities;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.data.Tuple;
import prefuse.data.tuple.BatchTupleSet;
import prefuse.visual.VisualItem;
import cytoPrefuse.CytoPrefuse;
import cytoPrefuse.EdgeBundler;

public class RubberBandSelect extends ControlAdapter  {
	private int downX1, downY1;
	private VisualItem rubberBand;
	Point2D screenPoint = new Point2D.Float();
	Point2D absPoint = new Point2D.Float();
	Rectangle2D rect = new Rectangle2D.Float();	
	Tuple[] selectedItems;
	public RubberBandSelect(VisualItem rubberBand) {
		this.rubberBand = rubberBand;
	}

	public void mousePressed(MouseEvent e) {    
		 if (!SwingUtilities.isLeftMouseButton(e)) return;
		 
		 Display d = (Display)e.getComponent();
		 d.setHighQuality(false);
		 screenPoint.setLocation(e.getX(), e.getY());
		 d.getAbsoluteCoordinate(screenPoint, absPoint);
		 downX1 = (int) absPoint.getX();
		 downY1= (int) absPoint.getY();
		 rubberBand.setVisible(true);
	 } 
	 
	 public void mouseDragged (MouseEvent e) {
		 if (!SwingUtilities.isLeftMouseButton(e)) return;
	
		 Display d = (Display)e.getComponent();			
		 screenPoint.setLocation(e.getX(), e.getY());
		 d.getAbsoluteCoordinate(screenPoint, absPoint);
		 int x1 = downX1;
		 int y1 = downY1;
		 int x2 = (int) absPoint.getX();
		 int y2 = (int) absPoint.getY();
		 	
		 float[] bandRect = (float[]) rubberBand.get(VisualItem.POLYGON);
		 bandRect[0] = x1;
		 bandRect[1] = y1;
		 bandRect[2] = x2;
		 bandRect[3] = y1;
		 bandRect[4] = x2;
		 bandRect[5] = y2;
		 bandRect[6] = x1;
		 bandRect[7] = y2;

		 if (x2 < x1){
			 int temp = x2;
			 x2 = x1;
			 x1 = temp;
		 }
		 if (y2 < y1) {
			 int temp = y2;
			 y2 = y1;
			 y1 = temp;
		 }
		 rect.setRect(x1,y1,x2-x1, y2-y1);
		 Visualization vis = d.getVisualization();
		 BatchTupleSet focus = (BatchTupleSet) vis.getFocusGroup(CytoPrefuse.SELECTED);
	
		 if (!e.isShiftDown()) {
			 //Try to avoid accidental rubber bands that are simple mouse clicks
			 if (rect.getWidth() * rect.getHeight() > 2.0) {			 
				 focus.clear();
				 //Some items might not be in the focus set (if they were selected
				 //through Cytoscape commands rather than user interaction.) Make sure
				 //we clear them all here.
				 CytoPrefuse cytoPrefuse = (CytoPrefuse)d;
				 cytoPrefuse.deselectAll();
			 }
		 }
		 selectedItems = new Tuple[vis.getGroup(CytoPrefuse.NODES).getTupleCount()];
		 Iterator it = vis.getGroup( CytoPrefuse.NODES).tuples();
		 
		 int i=0;
		 while (it.hasNext()) {
			 VisualItem item = (VisualItem) it.next();			 
			 if (item.isVisible() && item.getBounds().intersects(rect)){
				 selectedItems[i++] = item;
			 }
		 }
		 Tuple[]properlySizedSelectedItems = new Tuple[i];
		 System.arraycopy(selectedItems, 0, properlySizedSelectedItems,0, i);		 
		 focus.addBatch(properlySizedSelectedItems);
		
		 selectedItems = new Tuple[vis.getGroup(CytoPrefuse.EDGES).getTupleCount()];			
		 i=0;
		 it = vis.getGroup( CytoPrefuse.EDGES).tuples();
		 while (it.hasNext()) {
			 VisualItem item = (VisualItem) it.next();
			 if (item.isVisible() && item.getBounds().intersects(rect)){
				 //Bounding box intersect.  Now test precisely
				 if (EdgeBundler.edgeIntersectsRect(item, rect))
					 selectedItems[i++] = item;
			 }
		 }
		 properlySizedSelectedItems = new Tuple[i];
		 System.arraycopy(selectedItems, 0, properlySizedSelectedItems,0, i);		 
		 focus.addBatch(properlySizedSelectedItems);
			
	
			
		 vis.repaint();
	 }
	 
	 public void mouseReleased (MouseEvent e) {
		 if (!SwingUtilities.isLeftMouseButton(e)) return;
		 rubberBand.setVisible(false);
		 Display d = (Display)e.getComponent();			
		 d.setHighQuality(true);
		 d.getVisualization().repaint();
			
	 }
}
